@@ -1,3 +1,5 @@
+import pytest
+
 import ctypes
 
 from typing import Optional
@@ -101,6 +103,7 @@ def test_ggml_custom_op():
     ggml.ggml_free(ctx)
 
 
+@pytest.mark.skip(reason="ggml_quantize_q8_0 has been deprecated")
 def test_quantize():
     ne0 = 32
     ne1 = 1
@@ -109,6 +112,7 @@ def test_quantize():
     data_f32 = (ctypes.c_float * len(data))(*data)
     work = (ctypes.c_float * nelements)(0)
     hist = (ctypes.c_int64 * (1 << 4))(0)
+    # TODO: convert to ggml.ggml_quantize_chunk
     cur_size = ggml.ggml_quantize_q8_0(
         data_f32,
         ctypes.cast(work, ctypes.c_void_p),
